;*********************************************
;
; Include file for DDS driver routines
; AD9854 quadrature DDS
;
;********************************************

INITIALIZE_DDS:
	ldi	temp,0b00000000 
	out	DDS,temp			; Set up DDS port
	sbi	DDS,DDS_RESET			; Assert DDS master reset to initialize
	nop
	nop
	nop					; For at least 10 REFCLK cycles (10 MHz)
	nop					; Micro clock 4 MHz
	nop
	nop
	cbi	DDS,DDS_RESET
	sbi	DDS,IOR				; Reset serial interface
	nop
	nop
	cbi	DDS,IOR
;***************************
; Set up control register
;
;
;***************************
	ldi	temp_1,7		; Serial command word - Write to addr 7
	rcall	DATA2DDS		; (control reg) 
	ldi	temp_1,0b00000000	; Comp powered down, DACs,Digital active
					; MSByte in control reg 
	rcall	DATA2DDS	
	ldi	temp_1,0b00010100	; PLL range low, PLL active, Multiply by 20
					; 2nd Byte in control reg 
	rcall	DATA2DDS	
	ldi	temp_1,0b00000000	; QDAC source internal,mode 0 (single tone)
					; freq update external
	rcall	DATA2DDS	
	ldi	temp_1,0b01100000	; Inv sinc filter off,shaped keying enabled,MSB first
					; shape data ext, serial data out disabled					
					; 4th Byte in control reg 
	rcall	DATA2DDS	
	sbi	DDS,IOUD		; Update
	cbi	DDS,IOUD
	sbi	DDS,IOR			; Reset serial interface
	nop
	nop
	cbi	DDS,IOR
;
	ldi	temp_1,8		; Serial command word - Write to addr 8
					; Digital multiplier 
	rcall	DATA2DDS	
	ldi	temp_1,$FF		; Digital multiplier amplitude max
	rcall	DATA2DDS	
	ldi	temp_1,$FF		; Digital multiplier amplitude
					; 2nd Byte in control reg 
	rcall	DATA2DDS	
;
	ldi	temp_1,9		; Serial command word - Write to addr 9
	rcall	DATA2DDS	
	ldi	temp_1,$FF		; Multiplier max amplitude
	rcall	DATA2DDS	
	ldi	temp_1,$FF		; Digital multiplier amplitude
	rcall	DATA2DDS	
;
	sbi	DDS,IOUD
	cbi	DDS,IOUD
	ret				; return
;
;
;******************************************
; 
; Write new data to DDS
; Binary data at AtBinary + 0-5, MSB at 0
; DDS serial address in DDS_serial_address
;
;******************************************
UD_DDS:
	sbi	DDS,IOR			; Reset serial interface
	nop
	nop
	cbi	DDS,IOR
;
	lds	temp_1,DDS_serial_address; Get address data from SRAM	
	rcall	DATA2DDS		; write address to DDS	
	lds	temp_1,AtBinary		; Get data from SRAM
	rcall	DATA2DDS	;
;
	lds	temp_1,DDS_serial_address; Get address data from SRAM	
	cpi	temp_1,$0A		; ?O/P shape key ramp?
	breq 	UD_DDS_END		; If so then 1 byte only
;
	lds	temp_1,AtBinary+1	; Get data from SRAM next byte
	rcall	DATA2DDS	;
;
	lds	temp_1,DDS_serial_address; Get address data from SRAM	
	cpi	temp_1,$00		; ?Phase adjust reg 1 
	breq 	UD_DDS_END		; If so then 2 bytes only
	cpi	temp_1,$01		; ?Phase adjust reg 2 
	breq 	UD_DDS_END		; If so then 2 bytes only
	cpi	temp_1,$08		; ? I amplitude multiplier
	breq 	UD_DDS_END		; If so then 2 bytes only
	cpi	temp_1,$09		; ? Q amplitude multiplier
	breq 	UD_DDS_END		; If so then 2 bytes only
	cpi	temp_1,$0B		; ? Q DAC 
	breq 	UD_DDS_END		; If so then 2 bytes only
;
;
	lds	temp_1,AtBinary+2	; Get data from SRAM
	rcall	DATA2DDS	
	lds	temp_1,DDS_serial_address; Get address data from SRAM	
	cpi	temp_1,$06		; ?Ramp rate clock 
	breq 	UD_DDS_END		; If so then 3 bytes only
;
	lds	temp_1,AtBinary+3	; Get data from SRAM
	rcall	DATA2DDS	
;
	lds	temp_1,DDS_serial_address; Get address data from SRAM	
	cpi	temp_1,$05		; ?Update clock 
	breq 	UD_DDS_END		; If so then 4 bytes only
	cpi	temp_1,$01		; ?Control registers 
	breq 	UD_DDS_END		; If so then 4 bytes only
	lds	temp_1,AtBinary+4	; Get data from SRAM
	rcall	DATA2DDS		; All other registers 6 bytes
	lds	temp_1,AtBinary+5	; Get data from SRAM
	rcall	DATA2DDS	
;
UD_DDS_END:
	sbi	DDS,IOUD		; Update manually
	cbi	DDS,IOUD
	ret
;
;*************************
; Write a serial word to DDS
; bit counter in temp
; data in temp_1
;************************
DATA2DDS:
	ldi	temp,8			; bit counter
DATA2DDSLOOP:
	cbi	DDS,DDS_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	DDS,DDS_DATA
	cbi	DDS,DDS_CLOCK
	sbi	DDS,DDS_CLOCK		; Transfer data to DDS
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	DATA2DDSLOOP
	ret
;******************************************
;
; Routine to calculate 1 Hz steps 
; multiplies 32 bit binary data to give 48
; bit number binary data at AtBinary0-5
;
;************************************** 
CALC_1HZ:
	lds	mp32u0,AtBinary+3	; Multiply 32 bit data to 48 bits
	lds	mp32u1,AtBinary+2	; Setup registers for
	lds	mp32u2,AtBinary+1	;  multiply routine 	
	lds	mp32u3,AtBinary
	clr	mc32u3
	ldi	mc32u2,$15
	ldi	mc32u1,$79
	ldi	mc32u0,$8E		; Multiplier for 200.000000 MHz clock
					; to give 1 Hz steps
;
;***************************************************************************
;*
;* "mpy32u" - 32x32 Bit Unsigned Multiplication
;*
;* This subroutine multiplies the two 32-bit register variables 
;* mp32u3:mp32u2:mp32u13:mp32u0  and mc32u3:mc32u2:mc32u1:mc32u0..
;* The result is placed in mr2u7:mr32u6:mr32u5:mr32u4:mr32u3:mr32u2:mr32u1:mr32u0.
;* Note that the multiplier mp is placed in the low bytes of the result 
;*
;* 
;*
;***************************************************************************


;***** Code
	clr	mr32u7		;clear 4 highest bytes of result
	clr	mr32u6
	clr	mr32u5		;
	clr	mr32u4
	lsr	mp32u3		;rotate multiplier Low
	ror	mp32u2		;
	ror	mp32u1		
	ror	mp32u0		
;
	ldi	loop_counter,32	; 
MUL_LOOP:
	brcc	NOADD		; if carry set
	add	mr32u4,mc32u0	; add multiplicand Low to byte 2 of res
	adc	mr32u5,mc32u1	; add multiplicand high to byte 3 of res
	adc	mr32u6,mc32u2
	adc	mr32u7,mc32u3
NOADD:
	ror	mr32u7		;shift right result byte 7
	ror	mr32u6		;rotate right result byte 6
	ror	mr32u5		;etc
	ror	mr32u4		;
	ror	mr32u3		
	ror	mr32u2		
	ror	mr32u1		;
	ror	mr32u0		
	dec	loop_counter	; Loop
	brne	MUL_LOOP
;
	sts	AtBinary,mr32u5
	sts	AtBinary+1,mr32u4
	sts	AtBinary+2,mr32u3
	sts	AtBinary+3,mr32u2
	sts	AtBinary+4,mr32u1
	sts	AtBinary+5,mr32u0
	ret

;***********************************************
;
; Write to PLL chip
; National LMX2306/16/26
; Port A control port
; Bit 7		PLL_CLOCK	data clocks on pos transition
; Bit 6		PLL_DATA	data MSB first
; Bit 5		PLL_LE		data load enable to latch on HIGH
; PLL_INFO	3 bytes RAM		PLL variables
; MSB first, 3rd byte 5 bits only in MSBits 
;
;*************************************************
UD_PLL:
;
	lds	temp_1,PLL_INFO		; get first byte of data
	ldi	temp,8			; bit counter
UDPLL_LOOP1:
	cbi	PLL,PLL_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	PLL,PLL_DATA
	cbi	PLL,PLL_CLOCK
	sbi	PLL,PLL_CLOCK		; Transfer data to PLL
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	UDPLL_LOOP1
;
	lds	temp_1,PLL_INFO+1	; get next byte of data
	ldi	temp,8			; bit counter
UDPLL_LOOP2:
	cbi	PLL,PLL_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	PLL,PLL_DATA
	cbi	PLL,PLL_CLOCK
	sbi	PLL,PLL_CLOCK		; Transfer data to PLL
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	UDPLL_LOOP2
;
	lds	temp_1,PLL_INFO+2	; get third byte of data
	ldi	temp,5			; bit counter,5 bits only
UDPLL_LOOP3:
	cbi	PLL,PLL_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	PLL,PLL_DATA
	cbi	PLL,PLL_CLOCK
	sbi	PLL,PLL_CLOCK		; Transfer data to PLL
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	UDPLL_LOOP3
;
	sbi	PLL,PLL_LE		; latch data
	nop
	nop
	cbi	PLL,PLL_LE
	ret
;
;***************************************
;
; INITIALIZE PLL
; F1 RESET disabled
; F2 Power enabled
; F3-5 FoLD disabled
; F6 VCO polarity positive
; F7 Charge pump enabled
; F8 Fastlock diabled
; F9 Fastlock mode
; F10 Timeout counter disabled
; F11-14 Timeout counter
; F15-17 Test modes 0
; F18 Power up
;
;*********************************
INITIALIZE_PLL:
	ldi	temp,$00
	sts	PLL_INFO,temp
	ldi	temp,$02  
	sts	PLL_INFO+1,temp
	ldi	temp,$18  
	sts	PLL_INFO+2,temp
	sbi	PLL,PLL_LE		; clear LE
	cbi	PLL,PLL_LE
;
	lds	temp_1,PLL_INFO		; get first byte of data
	ldi	temp,8			; bit counter
INITPLL_LOOP1:
	cbi	PLL,PLL_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	PLL,PLL_DATA
	cbi	PLL,PLL_CLOCK
	sbi	PLL,PLL_CLOCK		; Transfer data to PLL
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	INITPLL_LOOP1
;
	lds	temp_1,PLL_INFO+1	; get next byte of data
	ldi	temp,8			; bit counter
INITPLL_LOOP2:
	cbi	PLL,PLL_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	PLL,PLL_DATA
	cbi	PLL,PLL_CLOCK
	sbi	PLL,PLL_CLOCK		; Transfer data to PLL
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	INITPLL_LOOP2
;
	lds	temp_1,PLL_INFO+2	; get third byte of data
	ldi	temp,5			; bit counter,4 bits only
INITPLL_LOOP3:
	cbi	PLL,PLL_DATA
	sbrc	temp_1,7		; If most sig bit set then set data bit
	sbi	PLL,PLL_DATA
	cbi	PLL,PLL_CLOCK
	sbi	PLL,PLL_CLOCK		; Transfer data to PLL
	lsl	temp_1			; Next bit
	dec	temp			; 
	brne	INITPLL_LOOP3
;
	sbi	PLL,PLL_LE		; latch data
	nop
	nop
	cbi	PLL,PLL_LE
	ret