;*****************************
; Global variables and equates
;*******************************
; Setup masks
;************
.equ	STACK_ADDRH	=$02		: Stack address top of SRAM
.equ	STACK_ADDRL	=$5F
.equ	INT_MASK	=0b11000000	; Mask for GIMSK INT1,INT0 interrupt enabled
.equ	TIMER_INT_MASK	=0b00000000	; Mask for timer interrupts (disabled)
.equ	MCUR_MASK	=0b00001000	; Mask for MCU Register Sleep off
					; INT1 triggered by rising edge
					; INT0 by low
.equ	TCCR0_MASK	=0b00000000	; Mask for timer 0 control reg (off)
.equ	TCCR1A_MASK	=0b00000000	; Mask for timer 1 control reg A(off)
.equ	TCCR1B_MASK	=0b00000000	; Mask for timer 1 control reg B(off)
.equ	TCCR2_MASK	=0b00000000	; Mask for timer 2 control reg (off)
.equ	ASSR_MASK	=0b00000000	; Asynchronous counter status reg 
.equ	DDRA_MASK	=0b11111111	; Port A all outputs
.equ	DDRB_MASK	=0b00111111	; Port B all outputs except bits 6,7
.equ	DDRC_MASK	=0b00001111	; Port C bits 0-3 outputs,4-7 inputs
.equ	DDRD_MASK	=0b00000000	; Port D all inputs
.equ	SPCR_MASK	=0b00000000	; SPI interface control reg
.equ	UCR_MASK	=0b00000010	; UART control reg
.equ	UBRR_MASK	=0b00000000	; UART baud rate reg
.equ	ACSR_MASK	=0b00000000	; Analaog comparator control reg
.equ	ADMUX_MASK	=0b00000000	; ADC Multiplexer control
.equ	ADCSR_MASK	=0b00000000	; ADC control reg
;******************
;Global variables
;*****************
.def    temp    	=r16    ;temporary register
.def    temp_1  	=r17
.def    temp_2 		=r18    ;temporary register
.def	temp_3		=r19
.def	temp_4		=r20
.def    temp_5 		=r21    ;temporary register
.def	temp_6		=r22
.def	temp_7		=r23
.def    temp_8  	=r24
.def    temp_9 		=r25    ;temporary register
.def	temp_10		=r26
.def	temp_11		=r27
.def    temp_12		=r28    ;temporary register
.def	temp_13		=r29

;*************************
; Subroutine Variables
;************************
; amplitude setting
.DSEG
amplitude:	.BYTE 1		; attenuator setting 
;bin2BCD24
;*********
.DSEG
AtBinary:	.BYTE 6		; Reserve 6 bytes for binary data
AtBCD:		.BYTE 6		; Reserve 6 bytes for BCD data
DDS_serial_address:	.BYTE 1	; reserve 1 byte for serial address
Sweep_data:	.BYTE 20	; variables for sweep data
PLL_INFO:	.BYTE 4		; PLL variables
 	

.CSEG
.equ	PLL		=$1b		; Port A PLL control
.equ	PLL_CLOCK	=7
.equ	PLL_DATA	=6		; Updates freq data in DDS
.equ	PLL_LE		=5		; Serial Clock DDS data
; Bit 7		PLL_CLOCK	data clocks on pos transition
; Bit 6		PLL_DATA	data MSB first
; Bit 5		PLL_LE		data load enable to latch on HIGH

.def	BCDMSB		=r22
.def	BCD3SB		=r23
.def	BCD2SB		=r24
.def	BCDLSB		=r25
.def    cnt16a  	=r20    ;loop counter
.def	tmp16a		=r21
.def	temp_LSB	=r19
.def    temp_2SB	=r18
.def	temp_3SB	=r17
.def    temp_MSB	=r16
;***** BCDadd Subroutine Register Variables

.def	BCD1add	=r21		;BCD input value #1
.def	BCD2add	=r22		;BCD input value #2
.def	tmpadd	=r23		;temporary register
;***** BCDsub Subroutine Register Variables

.def	BCD1sub	=r21		;BCD input value #1
.def	BCD2sub	=r22		;BCD input value #2

;***********
; BCD2BIN
;*******************
.def	adder		=r16		;value to add after multiplication	
.def	copyL		=r17		;temporary register
.def	copy2		=r18
.def	copy3		=r19		;temporary register
.def	copy4		=r20
.def	copy5		=r21
.def	copyH		=r22		;temporary register
.def	mp10L		=r23		;Low byte of number to be multiplied by 10
.def	mp102		=r24
.def	mp103		=r25		;Low byte of number to be multiplied by 10
.def	mp104		=r26
.def	mp105		=r27
.def	mp10H		=r28		;High byte of number to be multiplied by 10
;************************
; DDS
;**********************
.equ	DDS		=$1b		; Port A DDS control
.equ	DDS_RESET	=0		; DDS master reset bit 0
.equ	IOR		=1		; Resets serial interface of DDS
.equ	DDS_DATA	=2
.equ	IOUD		=4		; Updates freq data in DDS
.equ	DDS_CLOCK	=3		; Serial Clock DDS data

.equ	FSKHOL		=6
.equ	SHKEY		=7		; DDS key shaping bit
.equ	FREQ1		=$0090		; Address of freq word data #1
.equ	FREQ2		=$0098		; Freq word #2
;
;***********************
; DATA_DISPLAY
; COMMAND_DISPLAY
; INITIALIZE_DISPLAY
;***********************
.equ    OE		=0b00010000	; bit 4 port B display enable
.equ    RS		=0b00100000	; bit 5 in port B display register select
;***************************
;DISPLAY_FREQ
;**************************
.equ    AtBCDMSB 	=22    ;address (pointer)of BCDMSB
.equ    AtBCDLSB 	=25    ;address of BCDLSB
;**************************
; KEYPAD
;**************************
.DSEG
AtKEYPRESS:	.BYTE 1		; current keypress 
AtKEYFLAGS:	.BYTE 2		; Key status flags and number of current digit
AtKEYS:		.BYTE 10	; Keypresses
MODE:		.BYTE 1		; Operating mode
.CSEG
.def	status		=r20
.equ	AM		=1	; Bit 1 in mode data AM 
;********************************
;MUL32
;***** Subroutine Register Variables
.def	mc32u0		=r16		;multiplicand low byte
.def	mc32u1		=r17		;multiplicand 
.def	mc32u2		=r18		;multiplicand 
.def	mc32u3		=r19		;multiplicand high byte
.def	mp32u0		=r20		;multiplier low byte
.def	mp32u1		=r21		;multiplier 
.def	mp32u2		=r22		;multiplier 
.def	mp32u3		=r23		;multiplier high byte
.def	mr32u0		=r20		;result byte 0 (LSB)
.def	mr32u1		=r21		;result byte 1
.def	mr32u2		=r22		;result byte 2
.def	mr32u3		=r23		;result byte 3
.def	mr32u4		=r24		;result byte 4
.def	mr32u5		=r25		;result byte 5
.def	mr32u6		=r26		;result byte 6
.def	mr32u7		=r27		;result byte 7 (MSB)
.def	loop_counter	=r28

;************************
; ROTARY_ENCODER
;************************
.DSEG
rotary_encoder_flags:.BYTE 1	; Flags for encoder
rotary_encoder_count:.BYTE 1	; number of rotary encoder pulses
rotary_encoder_step: .BYTE 1	; Step size 	
.CSEG
.def    Delay    	=r16    ;temporary register
.def    Delay1   	=r17
.def    Delay2  	=r18    ;temporary register
;***********************
; WAIT_2msec
;***********************
.def    delay    	=r16    ;temporary register
.def    delay1   	=r17
.def    delay2  	=r18    ;temporary register
.def	delay3		=r19
;*********************************
;INT_SAVE
; data tables in RAM to save processor
;*********************************
.DSEG
status_reg:	.BYTE 1		; save status reg in RAM
registers:	.BYTE 32	; save all the working registers
.CSEG


