;*******************************
;
; DDSPLL board driver software main file
; Controller Atmel AT90S8535
; Analogue Devices AD9852 DDS
; National LMX2306 series PLL
;
; Richard Hosking 
; May 2002
;
;***********************************
.include	"8535def.inc"
.include	"DDSpldef.asm"
.include	"LCDfiles.asm"
.include	"keypad.asm"
.include	"DDSpfile.asm"
.include	"rotenc.asm"
.include	"INT_SAVE.asm"
.include	"SWEEP.asm"

;

.CSEG
.ORG	4000
AMPLITUDE_LOOKUP: 	; Lookup table
.DB $0F,$FF		; for amplitude
.DB $0E,$46		; in dB 00 to -60
.DB $0C,$B3
.DB $0B,$56
.DB $0A,$1A
.DB $09,$01
.DB $08,$07
.DB $07,$27
.DB $06,$60
.DB $05,$AE
.DB $05,$10
.DB $04,$84
.DB $04,$05
.DB $03,$95
.DB $03,$32
.DB $02,$DA
.DB $02,$89
.DB $02,$43
.DB $02,$04
.DB $01,$CC
.DB $01,$9A
.DB $01,$6D
.DB $01,$46
.DB $01,$22
.DB $01,$03
.DB $00,$E7
.DB $00,$CE
.DB $00,$B7
.DB $00,$A3
.DB $00,$91
.DB $00,$82
.DB $00,$74
.DB $00,$67
.DB $00,$5B
.DB $00,$52
.DB $00,$49
.DB $00,$41
.DB $00,$3A
.DB $00,$34
.DB $00,$2E
.DB $00,$29
.DB $00,$24
.DB $00,$21
.DB $00,$1D
.DB $00,$1A
.DB $00,$17
.DB $00,$15
.DB $00,$12
.DB $00,$10
.DB $00,$0F
.DB $00,$0D
.DB $00,$0B
.DB $00,$0A
.DB $00,$09
.DB $00,$08
.DB $00,$07
.DB $00,$06
.DB $00,$05
.DB $00,$04
.DB $00,$00  ;

.ORG	0


	rjmp	START			; Power on/hardware/WDT reset
	rjmp	KEYPAD			; Service keypad interrupt INT0
	rjmp	ROTARY_ENCODER		; Rotary encoder INT1

.ORG	50
START:
	ldi	temp,STACK_ADDRH	; Initialize High byte of 
	out	SPH,temp		; Stack Pointer  
	ldi	temp,STACK_ADDRL	; Low byte
	out	SPL,temp		; Stack pointer
	ldi	temp,TIMER_INT_MASK	; Enable timer interrupts			;
	out	TIMSK,temp
	ldi	temp,MCUR_MASK		; Mask for general MCU control register
	out	MCUCR,temp		; Sleep off, 
					; INT1 triggered by falling edge
					; INT0 by low
	ldi	temp,INT_MASK		; Global interrupt enable register			;
	out	GIMSK,temp		; Ext interuppts enabled (note this 
	ldi	temp,0b11000000		; Clear interrupt flag register
	out	GIFR,temp		; or interrupt will occur when sei					; should be done after setting up polarity
					; of interrupt
	ldi	temp,TCCR0_MASK		; Mask for timer 0 (off)	;
	out	TCCR0,temp
	ldi	temp,TCCR1A_MASK	; Timer 1 control reg A		;
	out	TCCR1A,temp	
	ldi	temp,TCCR1B_MASK			;
	out	TCCR1B,temp	
	ldi	temp,TCCR2_MASK			;
	out	TCCR2,temp	
	ldi	temp,ASSR_MASK
	out	ASSR,temp
	ldi	temp,0b00011111		; Set both WDDE (watchdog disable enable)
	out	WDTCR,temp		; and WDE (watchdog enable) in same operation	
	cbi	WDTCR,WDE		; Clear enable bit to disable WD timer
	ldi	temp,DDRA_MASK
	out	DDRA,temp
	ldi	temp,DDRB_MASK
	out	DDRB,temp	
	ldi	temp,DDRC_MASK
	out	DDRC,temp	
	ldi 	temp,$F0		;initialize keypad
	out 	PORTC,temp		; bits 0-3 outputs,low
					; Bits 4-7 inputs, pullups activated
	ldi	temp,DDRD_MASK
	out	DDRD,temp	
	ldi 	temp,0b0001110		;initialize PORTD so interrupts
	out 	PORTD,temp		; are not triggered (pullups activated)
	out	PIND,temp
	ldi	temp,SPCR_MASK
	out	SPCR,temp	
	ldi	temp,UCR_MASK
	out	UCR,temp
	ldi	temp,UBRR_MASK	
	out	UBRR,temp	
	ldi	temp,ACSR_MASK
	out	ACSR,temp
	sbi	ACSR,ACD		; Switch off analog comparator
	ldi	temp,ADMUX_MASK		; Analog to Digital convereter multiplexer
	out	ADMUX,temp   
	ldi	temp,ADCSR_MASK
	out	ADCSR,temp   
	rcall	DELAY_100ms	
	rcall	INITIALIZE_DDS
	rcall	INITIALIZE_DISPLAY
	rcall	INITIALIZE_PLL		; setup PLL 
;
;TMP:
;	rjmp	TMP
	ldi	temp,$00		; set PLL reference divider to 11
	sts	PLL_INFO,temp
	ldi	temp,$01  
	sts	PLL_INFO+1,temp
	ldi	temp,$60  
	sts	PLL_INFO+2,temp	
	rcall	UD_PLL			; 
;
	ldi	temp,$80		; set main divider to 1024
	sts	PLL_INFO,temp
	ldi	temp,$80  
	sts	PLL_INFO+1,temp
	ldi	temp,$88  
	sts	PLL_INFO+2,temp	
	rcall	UD_PLL			; 
;
	ldi	temp,$21		; Load an initial freq to DDS
	sts	AtBCD,temp		; 10.700000 MHz
	ldi	temp,$40	
	sts	AtBCD+1,temp		; 	
	ldi	temp,$00	
	sts	AtBCD+2,temp	
	ldi	temp,$00
	sts	AtBCD+3,temp	
	ldi	temp,0	
	sts	AtBCD+4,temp	
	ldi	temp,0
	sts	AtBCD+5,temp	
;
	rcall	BCD2BIN32
	rcall	DISPLAY_FREQ		; 
	rcall	CALC_1HZ		; Convert to 48 bits 1 Hz steps
	ldi	temp,$02		; DDS Freq 1 address 
	sts	DDS_serial_address,temp
	rcall	UD_DDS	

	clr	temp
	sts	AtKEYFLAGS,temp		; Clear KEYFLAGS
	ldi	temp,$01		;
	sts	rotary_encoder_step,temp; initialize step size 10 Hz
;
;	sei				; Enable interrupts
MAIN:
;	rjmp	MAIN
;	rcall	DELAY_100ms
;	rcall	DELAY_100ms
;	rcall	DELAY_100ms
	rcall	WAIT_2msec
	ldi	temp,$80		; set main divider to 1100
	sts	PLL_INFO,temp
	ldi	temp,$80  
	sts	PLL_INFO+1,temp
	ldi	temp,$88  
	sts	PLL_INFO+2,temp	
	rcall	UD_PLL		
	rjmp	MAIN
;	rcall	DELAY_100ms
;	rcall	DELAY_100ms
;	rcall	DELAY_100ms
	rcall	WAIT_2msec
	ldi	temp,$80		; set main divider to 1100
	sts	PLL_INFO,temp
	ldi	temp,$75  
	sts	PLL_INFO+1,temp
	ldi	temp,$88  
	sts	PLL_INFO+2,temp	
	rcall	UD_PLL			

;
	rcall	DISPLAY_FREQ		; 
;
	lds	temp,rotary_encoder_flags
	sbrs	temp,0			; ?rotary encoder serviced
	rjmp	CONT1
	clr	temp			; clear service flag
	sts	rotary_encoder_flags,temp
	rcall	BCD2BIN32
	rcall	DISPLAY_FREQ		; 
	rcall	CALC_1HZ		; Convert to 48 bits 1 Hz steps
	ldi	temp,$02		; DDS Freq 1 address 
	sts	DDS_serial_address,temp
	rcall	UD_DDS	
CONT1:
	lds	temp,AtKEYFLAGS		; Check for keypress flag
	sbrs	temp,4
	rjmp	CONT
	rcall	KEYPAD_HANDLER		; If so then deal
	rcall	BCD2BIN32		; Calculate new freq
	rcall	CALC_1HZ		; Convert to 48 bits 1 Hz steps
	ldi	temp,$02		; DDS Freq 1 address 
	sts	DDS_serial_address,temp
	rcall	UD_DDS	
CONT:
	lds	temp,MODE		; Get mode
	sbrc	temp,AM
	nop
	rjmp	MAIN
;
;*******************************************
; Subroutines
;
;******************************************
DELAY_100ms:
        ldi     Delay2,$05
        ldi     Delay3,$FF
D100ms:
        dec     Delay2
        brne    D100ms
        dec     Delay3
        brne    D100ms	
	ret
;	
;
;

;

