

;*******************************************
;
; Keypad service routine 
; Returns key pressed in SRAM at AtKEYS
; Current byte indicated by KEYFLAGS
; Keypad on Port C and INT0 (D2)
;
;******************************************
KEYPAD:
	ldi	temp,0			; Global interrupt enable register			;
	out	GIMSK,temp		; Ext interrrupts disabled 
					; have to do this, because we're
					; using a level-triggered interrupt
	rcall	SAVE_PROCESSOR		; Save processor registers because 
					; only the 
					; PC is saved automatically
;	ldi 	temp,0x00
;	out 	GIMSK,temp 		;disable external interrupt
	ldi	temp,6
KEY1:	
	rcall	WAIT_2msec		; Wait about 20 msec
	dec	temp			; to debounce key
	brne	KEY1
;
	ldi	temp,0b11111110		; Column 1 low
	out	PORTC,temp
	rcall	WAIT_2msec 		; Let line stabilize
	in	temp,PINC		; Any key pressed?
	cpi	temp,0b11111110
	brne	KEY2			; If yes then save
;
	ldi	temp,0b11111101		; Column 2 low
	out	PORTC,temp
	rcall	WAIT_2msec 	
	in	temp,PINC		; Any key pressed?
	cpi	temp,0b11111101
	brne	KEY2			; If yes then save	
;
	ldi	temp,0b11111011		; Column 3 low
	out	PORTC,temp
	rcall	WAIT_2msec 	
	in	temp,PINC		; Any key pressed?
	cpi	temp,0b11111011
	brne	KEY2			; If yes then save
;
	ldi	temp,0b11110111		; Column 4 low
	out	PORTC,temp
	rcall	WAIT_2msec 	
	in	temp,PINC		; Any key pressed?
	cpi	temp,0b11110111
	brne	KEY2			; If yes then save
KEY2:
	ldi	temp_1,$31		; Key for 1
	cpi	temp,0b11101110		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$32		; Key for 2
	cpi	temp,0b11101101		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$33		; Key for 3
	cpi	temp,0b11101011		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$41		; Key for A
	cpi	temp,0b11100111		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$34		; Key for 4
	cpi	temp,0b11011110		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$35		; Key for 5
	cpi	temp,0b11011101		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$36		; Key for 6
	cpi	temp,0b11011011		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$42		; Key for B
	cpi	temp,0b11010111		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$37		; Key for 7
	cpi	temp,0b10111110		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$38		; Key for 8
	cpi	temp,0b10111101		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$39		; Key for 9
	cpi	temp,0b10111011		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$43		; Key for C
	cpi	temp,0b10110111		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$2A		; Key for *
	cpi	temp,0b01111110		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$30		; Key for 0
	cpi	temp,0b01111101		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$23		; Key for #
	cpi	temp,0b01111011		; Lookup table for keypad
	breq	KEY3
	ldi	temp_1,$44		; Key for D
	cpi	temp,0b01110111		; Lookup table for keypad
	breq	KEY3
;	ldi	temp_1,0b00111111	; Error ?
KEY3:
	sts	AtKEYPRESS,temp_1	; Store key data
;
	lds	temp,AtKEYFLAGS		; Get Keyflags
	sbr	temp,$10		; Set key flag
	sts	AtKEYFLAGS,temp
	rcall	DELAY_100ms		; Wait or second keypress will be initiated
	rcall	DELAY_100ms
	rcall	DELAY_100ms		; Wait or second keypress will be initiated
	rcall	DELAY_100ms
;
	ldi	temp,0b11110000		; Restore columns 
	out	PORTC,temp
	ldi	temp,INT_MASK		; Global interrupt enable register			;
	out	GIMSK,temp		; Ext interrrupts enabled 
	rcall	RESTORE_PROCESSOR	; recover processor after interrupt
;
	reti				; Return
;**************************************
;
; Keypad handler
;
;************************************
;AtKEYPRESS 	current keypress 
;AtKEYFLAGS 	Key status flags and number of current digit
;AtKEYS		 Keypresses
;
KEYPAD_HANDLER:

;**********************************************************************
; Data entry routine
; Allows entry of:			code
;	Operating freq			any digit
;	Sweep				A
; 	Attenuator setting		B
; 	FM				C
; 	Rotary encoder step		D
; 					6
; 					7
; 					8
; 					9
;
;**********************************************************************
;

	clr	temp	
	sts	AtKEYFLAGS,temp		; Clear keypress flag
	lds	temp,AtKEYPRESS		; Get current keypress
	cpi	temp,$41		; ?A key
	breq 	A_KEY
	cpi	temp,$42		; ?B
	breq	B_KEY
	cpi	temp,$44		; ?D
	rjmp	OTHER_KEYS
	rjmp	OTHER_KEYS		; Capture any other keypresses
A_KEY:
	rcall	SWEEP			; Sweep module
B_KEY:
	rjmp	ATTEN			; Attenuator set mode
D_KEY:
	rjmp	ENCODER_STEP		; Set rotary encoder step	
OTHER_KEYS:
	rjmp	SINGLE_TONE		; All other digits single tone
;
ATTEN:
        ldi     temp,$01                ; Clear display
        rcall   COMMAND_DISPLAY
        rcall   WAIT_2msec
	ldi	temp,$0F		; Cursor on, flashing on
	rcall	COMMAND_DISPLAY
	rcall	WAIT_2msec
	ldi     temp,$C0        	; Cursor to 40
	rcall    COMMAND_DISPLAY	; Write to second line
        ldi     temp,$20                ; Blank
        rcall   DATA_DISPLAY
        ldi     temp,$41                ; A
        rcall   DATA_DISPLAY
        ldi     temp,$54                ; T
        rcall   DATA_DISPLAY
        ldi     temp,$54                ; T
        rcall   DATA_DISPLAY
        ldi     temp,$45                ; E
        rcall   DATA_DISPLAY
        ldi     temp,$4E                ; N
        rcall   DATA_DISPLAY
        ldi     temp,$20                ; Blank
        rcall   DATA_DISPLAY
        ldi     temp,$B0                ; -
        rcall   DATA_DISPLAY
;
	lds	temp,amplitude		; get current amplitude 
	swap	temp			; display most sig digit
	andi	temp,$0F		; mask upper nibble 
	sbr	temp,$30		; ASCII for digit
        rcall   DATA_DISPLAY		
	lds	temp,amplitude		; get current amplitude 
	andi	temp,$0F		; mask upper nibble 
	sbr	temp,$30		; ASCII for digit
        rcall   DATA_DISPLAY
        ldi     temp,$64                ; d
        rcall   DATA_DISPLAY
        ldi     temp,$42                ; B
        rcall   DATA_DISPLAY
	ldi     temp,$C8        	; Cursor to 48
	rcall    COMMAND_DISPLAY	; 
	sei				; enable keypad
ATT_LOOP1:
	lds	temp,AtKEYFLAGS		; Get Keyflags
	sbrs	temp,4			; ?keypress
	rjmp	ATT_LOOP1		; wait for keypress
;
	clr	temp			; clear keypress flag
	sts	AtKEYFLAGS,temp
	lds	temp,AtKEYPRESS		; Get current keypress
	andi	temp,$F0		; mask off lower 4 bits
	cpi	temp,$30		; is it a digit?
	brne	ATT_LOOP1		; if not get another digit
	lds	temp,AtKEYPRESS		; Get current keypress
	rcall	DATA_DISPLAY		; display
	lds	temp,AtKEYPRESS		; Get current keypress
	lds	temp_1,amplitude	; Get current amplitude
	swap	temp			; keypress to most sig 
	andi	temp,$F0		; digit of amplitude
	andi	temp_1,$0F
	add	temp_1,temp
	sts	amplitude,temp_1	; and store
ATT_LOOP2:
	lds	temp,AtKEYFLAGS		; Get Keyflags
	sbrs	temp,4			; ?keypress
	rjmp	ATT_LOOP2		; wait for keypress
;
	clr	temp			; clear keypress flag
	sts	AtKEYFLAGS,temp
	lds	temp,AtKEYPRESS		; Get current keypress
	andi	temp,$F0		; mask off lower 4 bits
	cpi	temp,$30		; is it a digit?
	brne	ATT_LOOP2		; if not get another digit
	lds	temp,AtKEYPRESS		; Get current keypress
	rcall	DATA_DISPLAY		; display
	lds	temp,AtKEYPRESS		; Get current keypress
	lds	temp_1,amplitude	; Get current amplitude
					; keypress to least sig 
	andi	temp,$0F		; digit of amplitude
	andi	temp_1,$F0
	add	temp_1,temp
	sts	amplitude,temp_1	; and store
	cli				; stop interrupts
	ldi	r31,HIGH(AMPLITUDE_LOOKUP*2)	; load address to Z
	ldi	r30,LOW(AMPLITUDE_LOOKUP*2)
;
	clr	temp	
	mov	temp_2,temp_1		; calculate address of amplitude
	andi	temp_1,$0F		; save copy of lower nibble
	swap 	temp_2			; Multiply upper nibble by 10
	andi	temp_2,$0F
	lsl	temp_2
	mov	temp_3,temp_2
	lsl	temp_2
	lsl	temp_2
	add	temp_2,temp_3		; add it all together
	add	temp_2,temp_1
	lsl	temp_2			; double address
	add	r30,temp_2		; and get offset
	clr	temp
	adc	r31,temp
	lpm				; lookup table
	sts	AtBINARY,r0		; save to RAM
	adiw	r30,1			; get upper byte
	lpm	
	sts	AtBINARY+1,r0		; to update DDS
	ldi	temp,$08		; address of amplitude mult 
	sts	DDS_serial_address,temp	; in DDS
	rcall	UD_DDS			; set new amplitude
	ldi	temp,$09		; address of amplitude mult 
	sts	DDS_serial_address,temp	; in DDS
	rcall	UD_DDS			; set new amplitude
	rjmp	REFEND
;
ENCODER_STEP:				
					; Set encoder step size
	lds	temp,AtKEYPRESS		; Get current keypress
	cpi	temp,$44		; ? is keypress D
	brne	SINGLE_TONE		; Set rotary encoder step
	lds	temp,rotary_encoder_step; get current step size
	inc	temp			; next step size
	cpi	temp,03			; If >100Hz then 1 Hz again
	brne	ENDS_R_S
	clr	temp
ENDS_R_S:
	sts	rotary_encoder_step,temp
	rjmp	REFEND			; end keypad handler
;	
SINGLE_TONE:
        ldi     temp,$01                ; Clear display
        rcall   COMMAND_DISPLAY
        rcall   WAIT_2msec
	ldi	temp,$0F		; Cursor on, flashing on
	rcall	COMMAND_DISPLAY
	rcall	WAIT_2msec
;*******************************
; Write the current entered data
; and get the required BCD data
;********************************

;
	lds	temp,AtKEYPRESS		; Get current keypress
	cpi	temp,$23		; ? is keypress #
	breq	REFEND			; if so then end
	sei				; enable interrupts
REFJUMP1:
	rcall	DISPLAY_FREQ		; Display current digits
;
	ldi	temp,$81		; Set cursor
	lds	temp_1,AtKEYFLAGS		
	andi	temp_1,$07		; Mask off all bit but digit count
	cpi	temp_1,2		; If more than 2 digits then skip *
	brlo	SKIP_STAR
	inc	temp_1
SKIP_STAR:
	add	temp,temp_1		; Adjust to current digit
	rcall	COMMAND_DISPLAY
;
WAIT_FOR_KEY:
	lds 	temp,AtKEYFLAGS		; wait for keypreess
        sbrs    temp,4          	; 
        rjmp    WAIT_FOR_KEY        	; 
;
	lds	temp,AtKEYPRESS		; Get current keypress
	cpi	temp,$23		; ? is keypress #
	breq	REFEND			; if so then end	
;
;******************************
; Calculate address of BCD data
;*****************************					
	ldi	temp_1,AtBCD		; address of BCD data
	lds	temp,AtKEYFLAGS		; Get current digit
	andi	temp,$07		; Mask off all but current digit
	mov	temp_2,temp		; make copy 
	asr	temp_2			; divide by 2
	add	temp_1,temp_2		; add to pointer to AtBCD
	mov	r30,temp_1		; Use Z as pointer
	clr	r31
;
	lds	temp,AtKEYFLAGS		; temp has keyflags - current digit
	ld	temp_1,Z		; temp_1 has current BCD digits
	lds	temp_2,AtKEYPRESS	; temp_2 has keypress
;
	sbrs	temp,0			; If even digit (0-7)
	rjmp	HIGH_NIBBLE		; then put keypress in high nibble
	andi	temp_1,$F0		; Otherwise store keypress
	andi	temp_2,$0F		; Mask off original digit
	rjmp	REFJUMP2
HIGH_NIBBLE:
	swap	temp_2			; then put keypress in high nibble
	andi	temp_1,$0F
	andi	temp_2,$F0		; Mask off original digit
REFJUMP2:
	add	temp_1,temp_2		; Store back BCD data
	st	Z,temp_1
;
	lds	temp,AtKEYFLAGS		; next digit
	inc	temp			; 
	sbrc	temp,3			; ?last digit
	andi	temp,$F0		; If so then reset to 0
	cbr	temp,$10		; Clear keypress flag
	sts	AtKEYFLAGS,temp		; and store back
;

        rjmp   	REFJUMP1       		; repeat
;
;

REFEND:
	sei				; enable interrupts
        ldi     temp,0b00001100 	; Display on, cursor/blink off
        rcall   COMMAND_DISPLAY
        rcall   WAIT_2msec
	clr	temp	
	sts	AtKEYFLAGS,temp		; Clear keypress flag		
	ret

;***************************************************************************
;*
;* "BCD2BIN32" - 8 digit BCD to 32-Bit Binary Conversion
;* This subroutine converts an 8-digit packed BCD number represented by 
;* 4 bytes (AtBCD in RAM) to a 4 byte binary number 
; (mp10H:mp105:mp104:mp103:mp102:mp10L).
;*  
;* Let "abcdefgh" denote the 8-digit number. The conversion is done by
;* computing the formula: 10(10(10(10(10(10(10(10(10a+b)+c)+d)+e)+f)+g)+h)+i)+j.
;* The subroutine "mul10a"/"mul10b" does the multiply-and-add operation 
;* which is repeated 7 times during the computation.
;* The result of the initial computation represents 1 Hz steps 
;* This is multiplied by $   to give a 48 bit number 
;* which can be sent to the DDS
;*  
;*
;***************************************************************************
;***** "mul10a"/"mul10b" Subroutine Register Variables

;***** Code
mul10a:	;***** multiplies "mp10H:mp10L" with 10 and adds "adder" high nibble 
	swap	adder
mul10b:	;***** multiplies "mp10H:mp10L" with 10 and adds "adder" low nibble 
	mov	copyL,mp10L	;make copy of binary data
	mov	copy2,mp102	;make copy	
	mov	copy3,mp103	;make copy
	mov	copyH,mp10H	;make copy
;	
	lsl	mp10L		;multiply original by 2
	rol	mp102
	rol	mp103
	rol	mp10H
	lsl	copyL		;multiply copy by 2
	rol	copy2
	rol	copy3
	rol	copyH		
	lsl	copyL		;multiply copy by 2 (4)
	rol	copy2
	rol	copy3
	rol	copyH		
	lsl	copyL		;multiply copy by 2 (8)
	rol	copy2
	rol	copy3
	rol	copyH		
	add	mp10L,copyL	;add copy to original
	adc	mp102,copy2
	adc	mp103,copy3
	adc	mp10H,copyH	
	andi	adder,0x0f	; mask away upper nibble of adder
	add	mp10L,adder	; add lower nibble of adder
	brcc	m10_1		; if carry not cleared
	inc	mp102		; inc middle bytes
	brne	m10_1		; and high byte if relevant
	inc	mp103
	brne	m10_1	
	inc	mp10H
m10_1:	ret	
;
;******************
; Main routine
;******************
BCD2BIN32:
	clr	mp10H		
	clr	mp103
	clr	mp102
	lds	mp10L,AtBCD	; mp10H:mp102:mp10L = a
	swap	mp10L		; High nibble to mp10L
	andi	mp10L,$0F	; Mask off other nibble
	lds	adder,AtBCD	; Other nibble to adder
	rcall	mul10b		; = 10a+b
	lds	adder,AtBCD+1
	rcall	mul10a		; = 10(10a+b)+c
	lds	adder,AtBCD+1
	rcall	mul10b		; = 10(10(10a+b)+c)+d
	lds	adder,AtBCD+2
	rcall	mul10a		; = 10(10(10(10a+b)+c)+d)+e
	lds	adder,AtBCD+2		
	rcall	mul10b		; = 10(10(10(10(10a+b)+c)+d)+e)+f
	lds	adder,AtBCD+3
	rcall	mul10a		; = 10(10(10(10(10(10a+b)+c)+d)+e)+f)+g
	lds	adder,AtBCD+3		
	rcall	mul10b		; = etc
;
	sts	AtBinary,mp10H	; Save result to RAM
	sts	AtBinary+1,mp103
	sts	AtBinary+2,mp102
	sts	AtBinary+3,mp10L
	ret

	
