;*************************************************************************
; Display setup
; Port B        bit             5       OE      Enable - clocks on neg transition
;                               4       RS      Register Select
;                                       command=0, data=1
;               bits    0-3     data    4 bits, high nibble first
;                                       ASCII format
;               RW grounded (write)     
; Note that commands take up to 1.6 msec for display to implement
; at a nominal display clock rate of 250 KHz
; Display initialize routine after power up
; 
;************************************************************************
.CSEG
.org	500
;
INITIALIZE_DISPLAY:
;
	ldi	temp,50			; Wait at least 15msec after
D1:					; powerup before writing 
	rcall	WAIT_2msec		; to display
	dec	temp
	brne	D1
        ldi     temp,0b00010011 	; System set 
        out     PORTB,temp
 ;       nop
 ;       nop                             ; Data write cycle must be >1000usec
        cbi     PORTB,4         	; OE low to clock in data
;
	rcall	WAIT_2msec		; Wait 4 msec
	rcall	WAIT_2msec
;
        ldi     temp,0b00010011          ; System set 
        out     PORTB,temp
 ;       nop
 ;       nop                             ; Data write cycle must be >1000usec
        cbi     PORTB,4         	; OE low to clock in data
;
	rcall	WAIT_2msec		; wait at least 100usec
;
        ldi     temp,0b00010011         ;System set
        out     PORTB,temp
 ;       nop                             ;
 ;       nop                                     ;
        cbi     PORTB,4         	; OE low to clock in data
;
        rcall   WAIT_2msec             	; Wait at least 40usec (2 msec)
;
        ldi     temp,0b00101000         ; Function set 
                                     	; 4 bit mode, 2 lines 5X7 pixels
        rcall   COMMAND_DISPLAY		;write to display
;
        ldi     temp,0b00001000         ;Display off, cursor off
                                        ; blink off
        rcall   COMMAND_DISPLAY
;
        ldi     temp,0b00000001         ; Display clear
        rcall   COMMAND_DISPLAY
        rcall   WAIT_2msec     		; Need to wait 1.6 msec after clear
;
        ldi     temp,0b00000110         ; Entry mode set
                                    	; Increment RAM, dont shift display
        rcall   COMMAND_DISPLAY
;
        ldi     temp,0b00001100		; Display on, cursor/blink off
        rcall   COMMAND_DISPLAY
	ret
;************************************************
;
; Write command to display
; command word in temp
; built in 50 usec delay
;
;************************************************
COMMAND_DISPLAY:
        mov     temp_1,temp     ; Copy data to temp_1
        andi    temp,0b11110000 ; mask off lower 4 bits
	swap	temp		; To correct for board error	
	cbr	temp,RS
        sbr     temp,OE         ; OE bit high
        out     PORTB,temp               ; write upper 4 bits to display
        nop                             ; wait 1 usec total command
        nop                                     ; Stabilize line
        cbi     PORTB,4         ; OE low to clock in data
;
        swap    temp_1  	; get lower 4 bits
        andi    temp_1,0b11110000       ; mask off lower 4 bits
	swap	temp_1		; To correct for board error
	cbr	temp_1,RS
        sbr     temp_1,OE       ; OE high
        out     PORTB,temp_1    ;write lower 4 bits to LCD                ;
        nop                             ; 
        nop                                     ; 
        cbi     PORTB,4         ; OE low to clock in data
;
        ldi     Delay,100       ; Wait about 50 usec   
D4:
	dec     Delay
        brne    D4
;
        ret     		; return
;
;******************************************************
;
; Writes data to display
; incorporates 50 usec delay
; data in temp
;
;********************************************************
DATA_DISPLAY:
        mov     temp_1,temp     ; Copy data to temp_1
        andi    temp,0b11110000 ; mask off lower 4 bits
	swap	temp		; Board error
        sbr     temp,RS         ; OE bit high
        sbr     temp,OE         ; data/command bit high
        out     PORTB,temp      ; write upper 4 bits to display
        nop                     ; wait 1 usec total command
        nop                                     ; Stabilize line
        cbi     PORTB,4         ; OE low to clock in data
;
        swap    temp_1  	; get lower 4 bits
        andi    temp_1,0b11110000       ; mask off lower 4 bits
	swap	temp_1		; Board error	
        sbr     temp_1,RS       ; OE high
        sbr     temp_1,OE
        out     PORTB,temp_1   ; write lower 4 bits to LCD                ;
        nop                             ; 
        nop                                     ; 
        cbi     PORTB,4         ; OE low to clock in data
;
        ldi     Delay,100       ; Wait about 50 usec   
D5  :
	dec     Delay
        brne    D5
;
        ret     		; return
;*********************************************************************
; Delay 2 msec with a 4MHz clock
;*********************************************************************
WAIT_2msec:
        ldi     Delay1,$0B     
D24002: 
        ldi     Delay2,$FF
D24001: dec     Delay2
        brne    D24001
        dec     Delay1
        brne    D24002
        ret
;
;******************************
;
; This routine writes the BCD data to the display
;
;*******************************

DISPLAY_FREQ: 
        ldi     temp,0b00000010 	; Cursor home
        rcall   COMMAND_DISPLAY
	rcall	WAIT_2msec 
        ldi     temp,$20                ; Blank
        rcall   DATA_DISPLAY
;
	ldi     temp_2,4                ; Loop counter
	lds	BCDMSB,AtBCD		; Get BCD data from RAM
	lds	BCD3SB,AtBCD+1
	lds	BCD2SB,AtBCD+2
	lds	BCDLSB,AtBCD+3
        ldi     r30,AtBCDMSB-1     	; Load ZL (pointer) with address of     
	clr	r31                	; tBCDMSB (Most Sig Byte) + predecrement
LOOP_DD:
        cpi    	temp_2,3        	; If third digit,then write a *
	brne   	MISS_STAR	      	; to separate MHz and KHz
        ldi     temp,$2A		; ASCII for *	
        rcall   DATA_DISPLAY
MISS_STAR:
        inc     r30
        ld      temp,Z         		;  MSByte of BCD data
        swap    temp
        andi    temp,0b00001111 	; Mask off upper 4 bits
        sbr     temp,$30                ; Add hex 30 to give ASCII
        rcall   DATA_DISPLAY
        ld      temp,Z         		; get next digit
        andi    temp,0b00001111
        sbr    	temp,$30
        rcall   DATA_DISPLAY    
        dec     temp_2
        brne    LOOP_DD 		; Repeat till all digits done
;
        ldi     temp,$20                ; Blank
        rcall   DATA_DISPLAY
        ldi     temp,$4D                ; M
        rcall   DATA_DISPLAY
        ldi     temp,$48                ; H
        rcall   DATA_DISPLAY
        ldi     temp,$7A                ; z
        rcall   DATA_DISPLAY
	ret
;
;******************************************************
;
; Writes contents of temp to display
; in hex format for debugging
; incorporates 50 usec delay
; data in temp
;
;********************************************************
DATA_DISPLAY_HEX:
        mov     temp_3,temp     ; Copy data to temp_1
	swap	temp		; Upper nibble first
	andi	temp,$0F	; Mask off upper bits
	cpi	temp,10		; If digit 9 or less then add 
	brsh	HEX1
	sbr	temp,$30	; $30 to display digit
	rjmp	HEX_DISPLAY1
HEX1:
	subi	temp,9		; Otherwise display A-F
	sbr	temp,$40
HEX_DISPLAY1:
	rcall	DATA_DISPLAY		
;
	mov	temp,temp_3
	andi	temp,$0F	; Mask off upper bits
	cpi	temp,10		; If digit 9 or less then add 
	brsh	HEX2
	sbr	temp,$30	; $30 to display digit
	rjmp	HEX_DISPLAY2
HEX2:
	subi	temp,9		; Otherwise display A-F
	sbr	temp,$40
HEX_DISPLAY2:
	rcall	DATA_DISPLAY		
	ldi	temp,$20
;	rcall	DATA_DISPLAY	
        ret     		; return